package com.ca.dso.itpam.webservice.client;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;


public class ItpamWebServiceClientFactory {
	
/**	Public string constant required by CA									*/
	public static final String CACOPYRIGHT = "(c) 2009 CA.  All rights reserved.";	
	
/**
 * Exception class instantiated and thrown when an exception is detected.
 * 																			*/
	public class ItpamWebServiceClientFactoryException extends Exception {
		
		private static final long serialVersionUID = -824642838757410184L;

	/** Constructor 														*/
		public ItpamWebServiceClientFactoryException(){
			
			super();
		}		
		
	/**
	 * Get exception message.
	 * 
	 * @return String set to exception message
	 * 																		*/
		public String getMessage(){
			
			return m_strExceptionMessage;
		}
	}
	
/** Private logger to record exception messages								*/	
	private static Logger logger = Logger.getRootLogger();
	
/**	Private string set to exception message									*/
	private String m_strExceptionMessage = "";
	
/** Private ITPAM web service client 										*/
	private ItpamWebServiceClient itpamWebServiceClient = null;
	
/**
 * Constructor
 */
	public ItpamWebServiceClientFactory(){}
	
	
/**
 * Instantiates and returns instance of appropriate web service based on
 * specified target end point version.
 *  
 * @param strTargetEndpointVersion - String set to target ITPAM version
 * 
 * @param strTargetEndpointURI - String set to URI of target ITPAM web service
 * 
 * @param strUser - String set to ITPAM user authorized to invoke required
 * web service methods
 * 
 * @param strPassword - String set ITPAM decrypted ITPAM user password
 * 
 * @return - ItpamWebServiceClient wrapper for version appropriate ITPAM web
 * service
 * 
 * @throws ItpamWebServiceClientFactoryException
 */
	public ItpamWebServiceClient getInstanceOf (String strTargetEndpointVersion,
									            String strTargetEndpointURI,
									            String strUser,
												String strPassword)
	throws ItpamWebServiceClientFactoryException
	{
	//	Disable logging if no appenders have been defined for the current root logger
		if (!ItpamWebServiceClientFactory.logger.getAllAppenders().hasMoreElements()) {
			ItpamWebServiceClientFactory.logger.setLevel(Level.OFF);
		}
		
	//	Record progress
		ItpamWebServiceClientFactory.logger.debug("Attemtpting to get instance of ITPAM web service client");	
		
	//	Instantiate appropriate version of client
		try {
			
			// The Target Version is now optional
			// 1. Retrieve the Version of the PAM server
			// 2. If user did not specify Version --> set the one retrieved and return it to the CLI
			// 	  If user specified Version	--> check that it's compatible with the one retrieved
			//								--> If compatible, then use the one specified by the user
			//								--> If not compatible, then return an error
			
			
			// Retrieve the Version of the PAM server
			// The response of the getITPamVersionInfo has changed from 3.0 to 4.X
			// hence we're trying to retrieve the version with either client
			String retrievedStrTargetEndpointVersion = "";
			ItpamWebServiceClient client = new ItpamWebServiceClient_3_0(strTargetEndpointVersion, strTargetEndpointURI, strUser, strPassword);
			try {
				retrievedStrTargetEndpointVersion = client.getITPamVersionInfo();
				ItpamWebServiceClientFactory.logger.debug("Retrieved PAM server version: " + retrievedStrTargetEndpointVersion);
			} catch (Exception ex) {
				ItpamWebServiceClientFactory.logger.debug("Unable to retrieve PAM server version with 3.0 client... Trying with 4.0 client.");
				client = new ItpamWebServiceClient_4_0(strTargetEndpointVersion, strTargetEndpointURI, strUser, strPassword);
				retrievedStrTargetEndpointVersion = client.getITPamVersionInfo();
				ItpamWebServiceClientFactory.logger.debug("Retrieved PAM server version: " + retrievedStrTargetEndpointVersion);
			}
			
			float retrievedFVersion = 0.0f;
			if ( retrievedStrTargetEndpointVersion != null){
				try{
					retrievedFVersion = Float.parseFloat(retrievedStrTargetEndpointVersion);
				}
				catch (NumberFormatException ex){
					retrievedFVersion = 0.0f;
				}
			}
			
			
			float fVersion = 0.0f;
			if ( strTargetEndpointVersion != null && strTargetEndpointVersion.length() > 0){
				ItpamWebServiceClientFactory.logger.debug("User specified PAM server version: " + strTargetEndpointVersion);
				try{
				fVersion = Float.parseFloat(strTargetEndpointVersion);
				}
				catch (NumberFormatException ex){
					fVersion = 0.0f;
				}
			} else {
				ItpamWebServiceClientFactory.logger.debug("User did not specify the PAM server version.");
				fVersion = retrievedFVersion;
				strTargetEndpointVersion = retrievedStrTargetEndpointVersion;
			}
			
			if(retrievedFVersion < fVersion) {
				// Set exception message
				this.m_strExceptionMessage = "The specified target end point version " + fVersion 
					+ " is not compatible with the retrieved PAM version " + retrievedFVersion;
				
				//	Log exception
				ItpamWebServiceClientFactory.logger.fatal(this.m_strExceptionMessage);
				
				//	Throw exception
				throw new ItpamWebServiceClientFactoryException();
			}
			
			//	Validate parameters
			this.validateParameters(strTargetEndpointVersion, strTargetEndpointURI, strUser, strPassword);
			
			// Versions 2.1 and 2.2 are no longer supported
			/*
			if (fVersion == 2.1){
				
				itpamWebServiceClient = new ItpamWebServiceClient_2_1(strTargetEndpointVersion,
																	  strTargetEndpointURI,
																	  strUser,
																	  strPassword);
			}
			else if (fVersion == 2.2){
				
				itpamWebServiceClient = new ItpamWebServiceClient_2_2(strTargetEndpointVersion,
						  											  strTargetEndpointURI,
																	  strUser,
																	  strPassword);
			}
			else
			*/
			
			if (( fVersion >= 3.0 ) && ( fVersion < 4.0)){
				
				itpamWebServiceClient = new ItpamWebServiceClient_3_0(strTargetEndpointVersion,
						  											  strTargetEndpointURI,
																	  strUser,
																	  strPassword);
			}
			else if ( fVersion >= 4.0  ){
				
				itpamWebServiceClient = new ItpamWebServiceClient_4_0(strTargetEndpointVersion,
						  											  strTargetEndpointURI,
																	  strUser,
																	  strPassword);
			}
			else {
				
			//	Set exception message
				this.m_strExceptionMessage = this.m_strExceptionMessage + "Target end point version ";
				this.m_strExceptionMessage = this.m_strExceptionMessage + "'" + strTargetEndpointVersion + "'";
				this.m_strExceptionMessage = this.m_strExceptionMessage + " not supported";
				
			//	Log exception
				ItpamWebServiceClientFactory.logger.fatal(this.m_strExceptionMessage);
				
			//	Throw exception
				throw new ItpamWebServiceClientFactoryException();
			}
		}
		catch (ItpamWebServiceClientFactoryException e){
			
		//	Throw exception
			throw e;
		}
		catch (Exception e){
			
		//	Set exception message
			this.m_strExceptionMessage = this.m_strExceptionMessage + "Unexpected exception '" + e.getMessage() + "'";
			this.m_strExceptionMessage = this.m_strExceptionMessage + " thrown in  " + Thread.currentThread().getStackTrace()[3].getClassName();
			this.m_strExceptionMessage = this.m_strExceptionMessage + "." +  Thread.currentThread().getStackTrace()[3].getMethodName();
			
		//	Log exception
			ItpamWebServiceClientFactory.logger.fatal(this.m_strExceptionMessage);
			
		//	Throw exception
			throw new ItpamWebServiceClientFactoryException();
		}
		
	//	Return instance of web service client
		return itpamWebServiceClient;
	}
	
	
/**
 * Inspect string specified to determine if null or empty.
 * 
 * @param strString - String to inspect
 * 
 * @return boolean true if null or empty otherwise false
 */
	private boolean isNullEmpty(String strString){
		
	//	Initialize boolean return value
		boolean bolNullEmpty = false;
		
	//	Reset boolean value if string is null or empty
		if (strString == null){
			
			bolNullEmpty = true;
		}
		else if (strString.equals("")){
			
			bolNullEmpty = true;
		}
		
		
	//	Return boolean value
		return bolNullEmpty;
	}
	
	
	/**
 * Add exception message string to exception messages vector if invalid
 * value specified for ITPAM user password.
 *  
 * @param strUser - String set to ITPAM user password
	 * @throws ItpamWebServiceClientFactoryException 
 * 																			*/
	private void validateParameter_Password (String strPassword)
	throws ItpamWebServiceClientFactoryException
	{

	//	Inspect parameter value
		if (this.isNullEmpty(strPassword)){
			
		//	Throw exception
			throw new ItpamWebServiceClientFactoryException();
		}
	}
	
/**
 * Add exception message string to exception messages vector if invalid
 * value specified for ITPAM target endpoint URI.
 *  
 * @param strUser - String set to ITPAM target endpoint URI
 * @throws ItpamWebServiceClientFactoryException 
 * 																			*/
	private void validateParameter_TargetEndPointURI (String strTargetEndpointURI)
	throws ItpamWebServiceClientFactoryException
	{
		
	//	Inspect parameter value
		if (this.isNullEmpty(strTargetEndpointURI)){
			
		//	Throw exception
			throw new ItpamWebServiceClientFactoryException();
		}
	}


/**
 * Add exception message string to exception messages vector if invalid
 * value specified for ITPAM target endpoint version.
 *  
 * @param strUser - String set to ITPAM target endpoint version
 * 
 * @throws ItpamWebServiceClientFactoryException 
 * 																			*/
	private void validateParameter_TargetEndPointVersion (String strTargetEndpointVersion)
	throws ItpamWebServiceClientFactoryException
	{
		//	Inspect parameter value
	
		if (this.isNullEmpty(strTargetEndpointVersion)){
			
		//	Throw exception
			throw new ItpamWebServiceClientFactoryException();
		}	
		
	}


/**
 * Add exception message string to exception messages vector if invalid
 * value specified for ITPAM user name.
 *  
 * @param strUser - String set to ITPAM user name
 * 
 * @throws ItpamWebServiceClientFactoryException 
 * 																			*/
	private void validateParameter_User (String strUser)
	throws ItpamWebServiceClientFactoryException
	{

	//	Inspect parameter value
		if (this.isNullEmpty(strUser)){
			
		//	Throw exception
			throw new ItpamWebServiceClientFactoryException();
		}	
	}


/**
 * Validate required parameter values.
 * 
 * @param strTargetEndpointVersion - String set to ITPAM target endpoint
 * version
 * 
 * @param strTargetEndpointURI - String set to ITPAM target endpoint URI
 * 
 * @param strUser - String set ITPAM user name
 * 
 * @param strPassword - String set ITPAM user password
 * 
 * @throws ItpamWebServiceClientFactoryException
 * 																			*/
	private void validateParameters (String strTargetEndpointVersion,
									 String strTargetEndpointURI,
									 String strUser,
									 String strPassword)
	throws ItpamWebServiceClientFactoryException
	{
	//	Initialize invalid parameter detected flag
		boolean bolInvalidParameterDetected = false;
		
	//	Validate target end point version
		try {
			
			this.validateParameter_TargetEndPointVersion(strTargetEndpointVersion);
		}
		catch (ItpamWebServiceClientFactoryException e){

			this.m_strExceptionMessage = this.m_strExceptionMessage + " 'version'";
			bolInvalidParameterDetected = true;
		}
		
	//	Validate target end point URI
		try {
			
			this.validateParameter_TargetEndPointURI(strTargetEndpointURI);
		}
		catch (ItpamWebServiceClientFactoryException e){
			
			this.m_strExceptionMessage = this.m_strExceptionMessage + " 'URI'";
			bolInvalidParameterDetected = true;
		}
		
	//	Validate user name
		try {
			
			this.validateParameter_User(strUser);
		}
		catch (ItpamWebServiceClientFactoryException e){

			this.m_strExceptionMessage = this.m_strExceptionMessage + " 'user name'";
			bolInvalidParameterDetected = true;
		}
		
	//	Validate user password
		try {
			
			this.validateParameter_Password(strPassword);
		}
		catch (ItpamWebServiceClientFactoryException e){

			this.m_strExceptionMessage = this.m_strExceptionMessage + " 'password'";
			bolInvalidParameterDetected = true;
		}
		
	//	Log and throw exception if an invalid parameter was detected
		if (bolInvalidParameterDetected){
			
		//	Set exception message
			this.m_strExceptionMessage = "Unable to instantiate ITPAM web service client:"
									   + " required parameter(s) missing or invalid"
									   + ";" + this.m_strExceptionMessage;
			
		//	Log exception
			ItpamWebServiceClientFactory.logger.fatal(this.m_strExceptionMessage);
			
		//	Throw exception
			throw new ItpamWebServiceClientFactoryException();
		}
	}
}
